"use strict";

var md5 = require('js-md5');

var localData = {
  data: {},
  setItem: function setItem(key, value) {
    this.data[key] = value;
  },
  clear: function clear() {
    this.data = {};
  },
  getItem: function getItem(key) {
    return this.data[key];
  },
  removeItem: function removeItem(key) {
    this.data[key] = null;
  }
};

var getStorage = function getStorage() {
  if (!window.localStorage) {
    return localData;
  } else {
    return window.localStorage;
  }
};

var TimeUnit = {
  SECONDS: 'SECONDS',
  MINUTES: 'MINUTES',
  HOURS: 'HOURS',
  DAYS: 'DAYS'
};
var LocalTimeUnit = {
  SECONDS: 1000,
  MINUTES: 1000 * 60,
  HOURS: 1000 * 60 * 60,
  DAYS: 1000 * 60 * 60 * 24
};

var removeItem = function removeItem(key) {
  if (typeof key !== 'string') {
    key = JSON.stringify(key);
  }

  getStorage().removeItem(md5(key));
};

var saveStartTime = function saveStartTime(startTime) {
  if (getStartTime() > parseInt("".concat(startTime || '0'))) {
    return;
  }

  getStorage().setItem('startTime', startTime);
};

var getStartTime = function getStartTime() {
  return parseInt(getStorage().getItem('startTime') || '0');
};

var saveData = function saveData(config) {
  if (config && config.key && config.url && config.data) {
    var url = config.url;
    var key = config.key;
    var cache = {
      url: url,
      key: key,
      persistent: config.persistent,
      startTime: getStartTime(),
      expires: !config.persistent ? new Date().getTime() + (config.expires || 3) * LocalTimeUnit[config.timeUnit || 'SECONDS'] : 0,
      data: config.data
    };
    getStorage().setItem(key, JSON.stringify(cache));
  }
};

var expires = function expires() {
  var storage = getStorage();

  for (var key in storage) {
    var item = storage.getItem(key);

    if (!item) {
      continue;
    }

    var cache = JSON.parse(item);

    if (!cache.persistent && cache.expires < new Date().getTime()) {
      getStorage().removeItem(key);
    }
  }
};

var getData = function getData(key, callback) {
  var cache = JSON.parse(getStorage().getItem(key));

  if (cache) {
    if (cache.startTime < getStartTime()) {
      getStorage().removeItem(key);
      return;
    }

    if (cache.persistent || cache.expires >= new Date().getTime()) {
      return new Promise(function (resolve, reject) {
        if (callback) {
          callback(cache.data);
        }

        resolve(cache.data);
      });
    }

    getStorage().removeItem(key);
  }
};

module.exports = function () {
  setInterval(expires, LocalTimeUnit.MINUTES);
  return {
    saveData: saveData,
    getData: getData,
    saveStartTime: saveStartTime,
    removeItem: removeItem
  };
};
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.saveCacheByUrl = saveCacheByUrl;
exports.getCacheByUrl = getCacheByUrl;

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var axios = require('axios');

var md5 = require('js-md5');

var _require = require('./cache')(),
    saveData = _require.saveData,
    saveStartTime = _require.saveStartTime,
    getData = _require.getData;

var timeoutUrl = {};

function saveCacheByUrl(value, data, urls) {
  var url = value.url || value;
  var config = urls[url] || {};

  if (config) {
    config.url = url;
    config.key = md5(JSON.stringify(value));
    config.data = {
      data: data
    };
    saveData(config);
  }
}

function getCacheByUrl(value, urls) {
  var config = urls[value.url] || {};

  if (config) {
    var data = getData(md5(JSON.stringify(value)));
    return data;
  }
}

var requestByCache = function requestByCache(_ref) {
  var config = _ref.config,
      request = _ref.request,
      urls = _ref.urls;
  var md = md5(JSON.stringify(config));

  if (timeoutUrl[md]) {
    if (!timeoutUrl[md].data) {
      return new Promise(function (resolve, reject) {
        reject('正在加载数据，请等待！！');
      });
    } else {
      delete timeoutUrl[md];
    }
  }

  timeoutUrl[md] = {};
  return (getCacheByUrl(config, urls) || request(config).then(function (response) {
    saveStartTime(response.headers['start-time']);
    saveCacheByUrl(config, response.data, urls);
    return response;
  })).then(function (response) {
    timeoutUrl[md].data = response.data;
    return response;
  }, function (error) {
    delete timeoutUrl[md];
    return error;
  })["catch"](function () {
    return delete timeoutUrl[md];
  });
};

module.exports = function (_ref2) {
  var baseURL = _ref2.baseURL,
      _ref2$timeout = _ref2.timeout,
      timeout = _ref2$timeout === void 0 ? 5000 : _ref2$timeout,
      _ref2$tokenKey = _ref2.tokenKey,
      tokenKey = _ref2$tokenKey === void 0 ? 'Authorization' : _ref2$tokenKey,
      urls = _ref2.urls,
      _ref2$requestIntercep = _ref2.requestInterceptor,
      requestInterceptor = _ref2$requestIntercep === void 0 ? function () {} : _ref2$requestIntercep,
      _ref2$getToken = _ref2.getToken,
      getToken = _ref2$getToken === void 0 ? function () {} : _ref2$getToken;
  var request = axios.create({
    baseURL: baseURL,
    timeout: timeout
  });
  request.interceptors.request.use(function (_ref3) {
    var config = _extends({}, _ref3);

    requestInterceptor(config);
    config.headers[tokenKey] = getToken();
    return config;
  }, function (error) {
    throw error;
  });
  request.interceptors.response.use(function (response) {
    return response;
  }, function (error) {
    var rejectObject = {
      error: error,
      type: 'error',
      message: '未定义的错误。'
    };

    if (!error.response) {
      rejectObject.message = '网络异常，请稍后刷新重试！';
      rejectObject.code = '1001';
    } else if (error.response.status === 401 || error.response.status === 403) {
      rejectObject.message = '无权访问！';
      rejectObject.code = '1002';
      rejectObject.type = 'low.power';
    } else {
      if (error.response.status === 502) {
        rejectObject.message = '网络繁忙，请稍候重试！';
      } else if (error.response && error.response.data && error.response.data.message) {
        rejectObject.message = error.response.data.message;
      } else if (error.message) {
        rejectObject.message = error.message;
      }
    }

    return Promise.reject(rejectObject);
  });
  return {
    request: request,
    finds: function finds(config) {
      if (urls) {
        return requestByCache({
          config: config,
          request: request,
          urls: urls
        });
      } else {
        return request(config);
      }
    },
    post: function post(config) {
      request(_objectSpread(_objectSpread({}, config), {}, {
        method: 'post'
      }));
    },
    put: function put(config) {
      request(_objectSpread(_objectSpread({}, config), {}, {
        method: 'put'
      }));
    },
    del: function del(config) {
      request(_objectSpread(_objectSpread({}, config), {}, {
        method: 'delete'
      }));
    },
    get: function get(config) {
      request(_objectSpread(_objectSpread({}, config), {}, {
        method: 'get'
      }));
    },
    getCacheData: getData,
    saveCacheData: saveData
  };
};
